/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

#include "motifR.h"
#include "Headers.h"


#define tempconstant 862.507157 /* conversion factor for the temperature */
#define energyconstant 1.19E-20 /* conversion factor for the energies */
#define vconstant 299.3793626

/* this routine will send the output to xmgr if there is nographics */

void XmgrHistFunctionNoGraphics(Variables* ptr)
{
  float hist_sum = 0;
  int i=0;

  /* we need to normalize the distribution */

  for(i=0;(*ptr).histogram[i] != 0;i++)
    hist_sum += (*ptr).histogram[i];
       
  for(i=0;(*ptr).histogram[i] !=0;i++)
    /* write the histogram */
    cout<<(i*(*ptr).binsize*vconstant)/*(sqrt(blennard))*/<<" "<<(*ptr).histogram[i]/hist_sum<<endl;   
  if( i > 0 )
  {
    cout<<"&"<<endl;
    cout<<"@title "<<"\""<<" Maxwell Bolzman distribution "<<"\""<<endl;
    cout<<"@subtitle "<<"\""<<" Probability of number particles at some velocity"<<"\""<<endl;
    cout<<"@ xaxis  label "<<"\""<<" Velocity (m/s)"<<"\""<<endl;
    cout<<"@ yaxis  label "<<"\""<<" Probability "<<"\""<<endl;
    cout<<"@ autoscale"<<endl;
    cout<<"@ kill s0"<<endl;
  }
}/* emd of the routine */

/* this routine will dump the the energies and other quantities */
void DumpQuantities(Variables* ptr,float instant_temp,float ipressure)
{
  char* time = new char[8];
  char* k_engy = new char[8];
  char* p_engy = new char[8];
  char* t_engy = new char[8];
  char* temp_str   = new char[8];
  char* press  = new char[8];
  char* itemp  = new char[8];
  char* ipress = new char[8];

  static Boolean reset = FALSE;

  if(!reset)
  { 
    cout<<"#Ctime  "<<"KE  "<<"PE  "<<"TE  "<<"TEMP  "<<"PRESS  "<<"ITEMP  "<<"IPRESS  "<<endl;
    reset = TRUE;
  }
  cout<<gcvt((*ptr).t,4,time)<<"     "<<gcvt((*ptr).kinetic_energy,4,k_engy)<<"     "<<
    gcvt((*ptr).potential_energy,4,p_engy)<<"     "<<gcvt((*ptr).total_energy,4,t_engy)<<"     "<<
    gcvt((*ptr).temp,4,temp_str)<<"     "<<gcvt((*ptr).total_pressure,4,press)<<"     "<<
    gcvt(instant_temp,4,itemp)<<"     "<<gcvt((*ptr).instant_pressure,4,ipress)<<endl;
  
  delete time;
  delete k_engy;
  delete p_engy;
  delete t_engy;
  delete temp_str;
  delete press;
  delete itemp;
  delete ipress;
}

/*********************************************************************
 *
 *
 *                 nographics routine
 *
 *
 *********************************************************************
 */

void NoGraphicsRoutine(Variables* ptr)
{ 
  static Boolean setstate = TRUE;
  static float avg_kinetic = 0;
  float avg_checker = avg_kinetic;
  float instant_temp = 0;
  float ipressure = 0;
  static unsigned int avg_counter = 0; 
  static float temp_interval = 0;
  static float graph_interval = 0;

  for(;;)
    { 
      if((*ptr).t < (*ptr).end_time)
	{
	  /* update positions */
	  (*ptr).t = update_position(ptr);

	  if((*ptr).mask == 11 && ((*ptr).t - graph_interval >= 20 ))
	  {
	    XmgrHistFunctionNoGraphics(ptr);
	    graph_interval = (*ptr).t;
	  }
	  if((*ptr).dump == 11)
	  {
	    /* energy calculations */
	    kinetic_energy_calc(ptr);
	    potential_energy_calc(ptr);    
	    (*ptr).total_energy = (*ptr).kinetic_energy + (*ptr).potential_energy;  
	    /* end of energy calculations */

	    /* calculate the integrated temperature */
	    avg_checker = avg_kinetic;
	    avg_kinetic += (double) ((*ptr).kinetic_energy * tempconstant)/((*ptr).nparticles /*blennard*/);
	    if(avg_checker != avg_kinetic)
	      avg_counter++;  
	    if((*ptr).t - temp_interval >= (*ptr).ftime_interval)
	    {	
	      (*ptr).temp = (avg_kinetic / avg_counter);
	      temp_interval = (*ptr).t;
	    } 
	    /* end of integrated temperature calculations */

	    instant_temp = ((*ptr).kinetic_energy * tempconstant)/((*ptr).nparticles /*blennard*/); 
	    DumpQuantities(ptr,instant_temp,ipressure);
	  }
	}
      else
	if(setstate)
	{
	  /* this will let the user know time is up */
	  //cout<<"# end_time = "<<(*ptr).end_time<<" but the system is till waiting for events "<<endl;
	  //cout<<"# you may type ^c to kill this proccess"<<endl;
	  cout<<"#end_time = "<<ptr->end_time;
	  setstate = FALSE;
	  exit(0);
	}

      /* reserved for interactivity, with the user 
      cin.getline(command_buffer,1024);
      nographics = CommandLineParser(argc,argv,ptr,nographics);*/
    }
}/* end of nographics routine */









